module JapanStations

export fetch_stations

using EzXML: EzXML, readxml

const LINE_TO_CODE = Dict{String,String}()

for (prefecture, code) in [("Tokyo", "13")]
    filepath = joinpath(@__DIR__, prefecture * ".xml")
    if !isfile(filepath)
        url = "http://www.ekidata.jp/api/p/$(code).xml"
        @info "Downloading $(url)."
        download(url, filepath)
    end
    @info "Loading $(basename(filepath))"
    doc = readxml(filepath)
    for line in findall("/ekidata/line", doc.root)
        lname = findfirst("line_name", line).content
        lcode = findfirst("line_cd", line).content
        LINE_TO_CODE[lname] = lcode
    end
end

const XML_CACHE = Dict{String,EzXML.Document}()

function fetch(url::AbstractString)
    if !haskey(XML_CACHE, url)
        XML_CACHE[url] = readxml(url)
    end
    return XML_CACHE[url]
end

"""
    fetch_stations(line) -> <a list of station names>

Fetch all the station names of `line`.
"""
function fetch_stations(line::AbstractString)
    code = get(LINE_TO_CODE, line, nothing)
    if code === nothing
        throw(ArgumentError("'$(line)' is not found"))
    end
    doc = fetch("http://www.ekidata.jp/api/l/$(code).xml")
    return [
        node.content for node in
        findall("/ekidata/station/station_name", doc)
    ]
end

end # module
